@@ -342,7 +342,11 @@ sub failure {
     # update session data about this form
     $self->to_session();
 
-    session(form_errors => '<ul>' . join('', map {"<li>$_</li>"} CORE::values %{$args{errors} || {}}) . '</ul>');
+    if (ref($args{errors}) eq 'ARRAY') {
+        $self->{errors} = {@{$args{errors}}};
+    }
+
+    session(form_errors => '<ul>' . join('', map {"<li>$_</li>"} CORE::values %{$self->{errors} || {}}) . '</ul>');
 
     session(form_data => $args{data});
 
